@echo off
set ver=v.1.7
:: ************************************************************************************
:: *                                                                                  *
:: ************************************************************************************
:: *                                                                                  *
:: *  Programmname: add_fixlease.cmd                                                  *
:: *                                                                                  *
:: *  Beschreibung: Erstellt beim DHCP-Server Reservierungen (fixe Leases).           *
:: *                Daten werden aus "fixlease_list.txt" entnommen.                   *
:: *                (Eine Kopiervorlage als Muster ist am Ende der Datei)             *
:: *                                                                                  *
:: *                                                                                  *
:: *  Version:      v.1.0                                                             *
:: *                v.1.1  erkennt auch Server 2008                                   *
:: *                v.1.2  erkennt auch Server 2008 R2                                *
:: *                v.1.3  erkennt auch Server 2012                                   *
:: *                v.1.4  Fehler bei Pfad korrigiert                                 *
:: *                v.1.5  erkennt auch Server 2012 R2, Versionsinfo korrigiert       *
:: *                v.1.6  erkennt auch Server 2016, Versionsinfo korrigiert          *
:: *                v.1.7  erkennt auch Server 2019, Versionsinfo korrigiert          *
:: *                                                                                  *
:: *  Datum:        2010-05-13                                                        *
:: *                2014-05-27                                                        *
:: *                2016-05-17                                                        *
:: *                2018-05-07                                                        *
:: *                                                                                  *
:: *  Author:       Helmut Pajnik                                                     *
:: *                IT-Regionalbetreuer Bregenz Nord                                  *
:: *                mail:  pajnik@vobs.at					      *
:: *                Tel : 0664-6255684                                                *
:: *            							              *
:: *  Bemerkung:    Programm basiert auf den Versionen von R.Sandholzer und E.Vonach. *
:: *                Die Datendatei "fixlease_list.txt" ist gleich aufgebaut und eine  *
:: *                schon bestehende kann uebernommen werden.                         *
:: *                                                                                  *
:: *                                                                                  *
:: ************************************************************************************
::
::    1a) Die Server-Variablen fuer DHCP-Server und -Bereich bitte anpassen
::
::    1b) Die Reservierungsdaten werden in der Textdatei "fixlease_list.txt" erwartet.
:: 
::    2)  Hostname, IP-Adresse, Kommentar und MAC-Adresse werden aus Datei ausgelesen.
::
::    3)  Bei MAC-Adresse "000000000000" wird versucht, die richtige MAC-Adresse zu ermitteln:
::
::        - Rechnername in IP-Adresse aufgelsen (NSLOOKUP)
::        - Warnung, wenn im DNS noch ein PC die gleiche IP hat (REVERSE LOOKUP)
::          Oft sind im DNS unsaubere Reste und Doppeleintraege!
::        - dann durch Abfrage der Adressleases des DHCP-Servers 
::          die dazugehrige MAC-Adresse des PCs ermitteln. (NETSH)
::        - Der PC muss dazu schon eine Lease zugewiesen bekommen haben! 
::          (etwa beim vorausgegangenen Klonen mit Ghost)    
::
::    4)  Eintragung der "Reservierung" im DHCP-Server
::
::    5)  Logfiles "newlog.txt" (=aktuelles) und "fixlease-log.txt" (=alle)  im TEMP
::
:: ------------------------------------------------------------------------------------
::
::     Voraussetzungen:
::     
::     - Windows 2000/2003/2008/2008R2/2012 Server als DHCP-Server
::     - Angemeldeter Benutzer muss Mitglied der Gruppe "DHCP-Benutzer" auf dem
::       DHCP-Server sein, um die DHCP-Daten auslesen zu knnen.
::     - Die Servervariablen unter "Server-Variable setzen" muessen stimmen! 
::       (Bitte die FOLGENDEN 2 VARIABLEN ANPASSEN !!!)  
::
:: ************************************************************************************

setlocal ENABLEEXTENSIONS



::**************************************************************************************
:: !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
:: ---SERVER-VARIABLE SETZEN------------------------------------------------------------
:: !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
::--------------------------------------------------------------------------------------
::
set DHCPSERVER=192.168.100.200
:: IP-Adresse oder Name des Servers (bei Name Aufruf mit : \\Name)
set DHCPBEREICHSNAME=192.168.100.0
:: set DHCPBereichsname=schule
:: --- "schule" geht NICHT, obwohl im DHCP-Server als Bereich angezeigt !!
::
::--------------------------------------------------------------------------------------
:: !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
::**************************************************************************************








:: ************************************************************************************
:: **   STARTMELDUNGENEN                                                             **
:: **                                                                                **
:: ************************************************************************************
::
color F0
cls
echo ==============================================================================
echo AUTOMATISIERTE EINTRAGUNG VON RESERVIERUNGEN IM DHCP  %ver%  -  H. Pajnik (rb)
echo ==============================================================================



:: ************************************************************************************
:: **   VERZEICHNISSE PRUEFEN - VARIABLE SETZEN                                      **
:: **                                                                                **
:: ************************************************************************************
::
:: ------------Aktuelles Verzeichnis ermitteln----------
pushd %~dp0
::                Verzeichnis, in dem die Batch aufgerufen wurde
set STARTDIR=%CD%
::
:: ------------Pfad korrigieren-------------------------
if "%startdir:~-1%"=="\" set startdir=%startdir:~0,-1%
:: Backslash am Ende wegschneiden
:: -----------------------------------------------------
set WORKDIR=%TEMP%
set LOGDIR=%TEMP%
::  -- hier habe ich sicher Schreibrechte ---
set LOGFILE=%LOGDIR%\fixlease-log.txt
set TEMPLOG=%LOGDIR%\templog.txt
set ERRORLOG=%LOGDIR%\errorlog.txt
set ERRORLOG2=%LOGDIR%\errorlog2.txt
set NEWLOG=%LOGDIR%\newlog.txt
set HOSTLOG=%LOGDIR%\hostlog.txt
REM -- erfolgreiche Hosts, Hosts mit Fehler und bereits eingetragene
set LOG_SUCCESS=%LOGDIR%\logsuccess.txt
set LOG_FAILED=%LOGDIR%\logFailed.txt
set LOG_EXIST=%LOGDIR%\logExist.txt
REM --
set TEMPDATEI1=%TEMP%\%~n0-tmp1.txt
set TEMPDATEI2=%TEMP%\%~n0-tmp2.txt
::zweite tmp noetig, da es sonst nicht immer funktioniert hat
set DELAY=2
::
REM -- DATEI mit DHCP-Liste-----------
set myFile="%STARTDIR%\fixlease_list.txt"
:: Datei mit DHCP-Steuerinfos 
:: -----------------------------------------------------







:: ---Betriebssystem-Version ermitteln----------------------------------------------------
::
set WIN_VER="unknown"
ver | find /i /c "Version 5.0" > NUL
if not ERRORLEVEL 1 set WIN_VER="Server 2000"
ver | find /i /c "Version 5.1" > NUL
if not ERRORLEVEL 1 set WIN_VER="Windows XP"
ver | find /i /c "Version 5.2" > NUL
if not ERRORLEVEL 1 set WIN_VER="Server 2003"
ver | find /i /c "Version 6.0" > NUL
if not ERRORLEVEL 1 set WIN_VER="Server 2008"
ver | find /i /c "Version 6.1" > NUL
if not ERRORLEVEL 1 set WIN_VER="Server 2008 R2"
ver | find /i /c "Version 6.2" > NUL
if not ERRORLEVEL 1 set WIN_VER="Server 2012"
ver | find /i /c "Version 6.3" > NUL
if not ERRORLEVEL 1 set WIN_VER="Server 2012 R2"
::echo Betriebssystem ist %WIN_VER%
ver | find /i /c "Version 10.0" > NUL
if not ERRORLEVEL 1 set WIN_VER="Server 2016"
::echo Betriebssystem ist %WIN_VER%
ver | find /i /c "Version 10.0" > NUL
if not ERRORLEVEL 1 set WIN_VER="Server 2019"
::echo Betriebssystem ist %WIN_VER%

:: ---------------------------------------------------------------------------------------



:: -------------------------------------------------------------------------------------
::
::=====LOG==============================================================================
echo =======================================================================  >"%TEMPLOG%"
echo PROTOKOLL: EINTRAGUNG VON RESERVIERUNGEN IM DHCP      %VER% - H. Pajnik >>"%TEMPLOG%"
echo ======================================================================= >>"%TEMPLOG%"
echo.                                                                        >>"%TEMPLOG%"
echo Programmstart am  %date%  um  Zeit:  %time%                             >>"%TEMPLOG%"
echo ----------------------------------------------------------------------- >>"%TEMPLOG%"
echo.                                                                        >>"%TEMPLOG%"
echo Betriebssystem ist %WIN_VER%                                            >>"%TEMPLOG%"
echo Benutzername ist "%USERNAME%"                                           >>"%TEMPLOG%"
::=====LOG==============================================================================
type "%TEMPLOG%" >> "%LOGFILE%"
type "%TEMPLOG%"  > "%NEWLOG%"
::



:: ---Betriebssystem-Version pruefen----------------------------------------------------
if NOT "%WIN_VER:~1,6%" =="Server" goto ErrorNoServer
:: 
:: --------------------------------------------------------------------------------------



::=====LOG==============================================================================
echo.                                                                   >"%TEMPLOG%"
echo ----------------------------------------------------------------- >>"%TEMPLOG%"
echo Einstellungen                                                     >>"%TEMPLOG%"
echo ----------------------------------------------------------------- >>"%TEMPLOG%"
echo --^>  DHCP-Server       : %DHCPSERVER%                            >>"%TEMPLOG%"
echo --^>  DHCP-Bereichsname : %DHCPBEREICHSNAME%                      >>"%TEMPLOG%"
echo --^>  Reservierungsliste: %MYFILE%                                >>"%TEMPLOG%"
echo.                                                                  >>"%TEMPLOG%"
::=====LOG==============================================================================
type "%TEMPLOG%" >> "%NEWLOG%"
type "%TEMPLOG%" >> "%LOGFILE%"
type "%TEMPLOG%"

::
::--warte DELAY  Sekunden--
ping -n %DELAY%  127.0.0.1 > NUL
::
:: ************************************************************************************


:: --Logfiles vorbereiten



echo --------------------------------------     >"%LOG_FAILED%"
echo FEHLER beim Eintragen folgender HOSTS:    >>"%LOG_FAILED%"
echo --------------------------------------    >>"%LOG_FAILED%"

echo --------------------------------------     >"%LOG_SUCCESS%"
echo NEU EINGETRAGENE HOSTS               :    >>"%LOG_SUCCESS%"
echo --------------------------------------    >>"%LOG_SUCCESS%"

echo --------------------------------------     >"%LOG_EXIST%"
echo Bereits ZUVOR VORHANDENE HOSTS       :    >>"%LOG_EXIST%"
echo --------------------------------------    >>"%LOG_EXIST%"



::--- Inhalt der Datei zerlegen und Unterprogramme aufrufen----------------

if not exist %myfile% goto ErrorNoFile
::
FOR /F "tokens=1,2,3* delims=;" %%G IN ('more ^<%myfile%') DO CALL :ADDFIXLEASE %%G %%H %%I %%J 

echo ----- DER JOB IST ERLEDIGT -------------------------------------------

::--warte DELAY  Sekunden--



::=====LOG==============================================================================
echo.                                                                   >"%TEMPLOG%"
echo.                                                                  >>"%TEMPLOG%"
echo ================================================================= >>"%TEMPLOG%"
echo ===                   ZUSAMMENFASSUNG                         === >>"%TEMPLOG%"
echo ================================================================= >>"%TEMPLOG%"
::=====LOG==============================================================================
type "%TEMPLOG%" >> "%NEWLOG%"
type "%TEMPLOG%" >> "%LOGFILE%"
echo.            >> "%LOGFILE%"



echo. >>"%LOG_SUCCESS%"
type    "%LOG_SUCCESS%" >> "%NEWLOG%"
type    "%LOG_SUCCESS%" >> "%LOGFILE%"

echo. >>"%LOG_EXIST%"
type    "%LOG_EXIST%" >> "%NEWLOG%"
type    "%LOG_EXIST%" >> "%LOGFILE%"

echo. >>"%LOG_FAILED%"
type    "%LOG_FAILED%" >> "%NEWLOG%"
type    "%LOG_FAILED%" >> "%LOGFILE%"

echo. >> "%NEWLOG%"
echo. >> "%NEWLOG%"
echo. >> "%LOGFILE%"
echo. >> "%LOGFILE%"


::=====LOG==============================================================================
echo -------------------------------------                              >"%TEMPLOG%"
echo Logfiles bei Bedarf selbst speichern                              >>"%TEMPLOG%"
echo -------------------------------------                             >>"%TEMPLOG%"
echo - aktuelles Log  : %NEWLOG%                                       >>"%TEMPLOG%"
echo - alle bisherigen: %LOGFILE%                                      >>"%TEMPLOG%"
echo.                                                                  >>"%TEMPLOG%"
echo ----------------------------------------------------------------- >>"%TEMPLOG%"
echo Eintragung beendet am  %date%  um  Zeit:  %time%                  >>"%TEMPLOG%"
echo ----------------------------------------------------------------- >>"%TEMPLOG%"
echo.                                                                  >>"%TEMPLOG%"
echo.                                                                  >>"%TEMPLOG%"
echo.                                                                  >>"%TEMPLOG%"
::=====LOG==============================================================================
type "%TEMPLOG%" >> "%NEWLOG%"
type "%TEMPLOG%" >> "%LOGFILE%"
echo.            >> "%LOGFILE%"


ping -n %DELAY%  127.0.0.1 > NUL
:: Pause noetig, damit Schreibvorgang genug Zeit hat
if exist %NEWLOG% (
  start "Logfile der Leaseeintragungen" /B "%ProgramFiles%\Internet Explorer\IEXPLORE.EXE" "%NEWLOG%"
)


::--aufraeumen--
echo. >%Tempdatei1%
echo. >%Tempdatei2%
echo. >%TEMPLOG%
echo. >%ERRORLOG%
echo. >%ERRORLOG2%
echo. >%HOSTLOG%
echo. >%LOG_SUCCESS%
echo. >%LOG_FAILED%
echo. >%LOG_EXIST%

del %Tempdatei1%
del %Tempdatei2%
del %TEMPLOG%
del %ERRORLOG%
del %ERRORLOG2%
del %HOSTLOG%
del %LOG_SUCCESS%
del %LOG_FAILED%
del %LOG_EXIST%

set WORKDIR=
set LOGDIR=
set LOGFILE=
set TEMPLOG=
set NEWLOG=
set HOSTLOG=
set LOG_SUCCESS=
set LOG_FAILED=
set LOG_EXIST=
set ERRORLOG=
set ERRORLOG2=
set TEMPDATEI1=
set TEMPDATEI2=
set DELAY=


endlocal


pause


goto eof


:: ----- ENDE DES HAUPTPROGRAMMES -------------------------------------------











:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:ADDFIXLEASE 
::
:: Func: Fuegt einen Reservierungs-Eintrag in die dhcp-Datenbank ein
::       Server 2000/2003.
:: 
:: Args: %1 MAC-Adresse
::       %2 IP-Adresse
::       %3 Hostname
::       %4 Kommentar
::   
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
setlocal 
set NEWMAC=000000000000
set IP=%2
set MAC=%1
set HOSTNAME=%3
SET KOMMENTAR=%4
if "%KOMMENTAR%"=="" set KOMMENTAR=%HOSTNAME%
if "%MAC:~0,1%"=="*" goto EXIT_ON_COMMENT  
:: * ist Kommentar
if "%MAC:~0,1%"=="#" goto EXIT_ON_COMMENT 
:: # ist Kommentar
set RES_STATUS="LEER"

::=====LOG==============================================================================
echo.                                                                   >"%TEMPLOG%"
echo ----------------------------------------------------------------- >>"%TEMPLOG%"
echo ======= bearbeite HOST - %HOSTNAME% =======                       >>"%TEMPLOG%"
echo ----------------------------------------------------------------- >>"%TEMPLOG%"
echo -- Quellzeile: MAC-%MAC%  IP-%IP%  HOST-%HOSTNAME%      >>"%TEMPLOG%"
::=====LOG==============================================================================
type "%TEMPLOG%" >> "%NEWLOG%"
type "%TEMPLOG%" >> "%LOGFILE%"
type "%TEMPLOG%" >> "%HOSTLOG%"
type "%TEMPLOG%"
del "%TEMPLOG%"


if NOT "%MAC%"=="000000000000" goto SETLEASE 
call :ASKMAC %HOSTNAME%  NEWMAC 
if NOT ERRORLEVEL 1 (
  set MAC=%NewMac%
  echo -- %Hostname%: ermittelte MAC:  %newmac% >>"%TEMPLOG%"
) 

:SETLEASE
if "%MAC%"=="000000000000" (
        REM ---
        REM Hinweis: nach 1x ausfuehren des Skripts ist im DNS noch die alte IP, 
        REM die neue Reservierung jedoch eingetragen. Damit an dieser Stelle keine Fehlermeldung erscheint,
        REM wird im DHCP ueberprft, ob die Reservierung schon eingetragen ist
        REM ---
        netsh dhcp server \\%DHCPServer% scope %DHCPBereichsname% show clients 1 | find /i "%IP%" >%Tempdatei2%
        if not errorlevel 1 (
          type %Tempdatei2%|find /i "%HOSTNAME%" >NUL
          if not errorlevel 1  (
            echo -- RESERVIERUNG VORGEMERKT: %HOSTNAME% mit %IP% ^(inaktiv^) >>"%TEMPLOG%"
            set RES_STATUS="INACTIVE" 
            goto Ausstieg
          )      
        )
        echo !! PROBLEM: Kann keine MAC-Adresse fuer %HOSTNAME% ermitteln >>"%TEMPLOG%"
        echo    Vorschlag: Ziel-PC einschalten und Skript erneut ausfuehren >>"%TEMPLOG%"
        set RES_STATUS="FEHLER"
        goto Ausstieg
)

echo -- Reserviere: %HOSTNAME% mit %IP% und %MAC%. >>"%TEMPLOG%"
REM  netsh dhcp server 192.168.100.200 scope 192.168.100.0 add reservedip %IP% %MAC% %HOSTNAME% "%KOMMENTAR%" "DHCP"
netsh dhcp server \\%DHCPSERVER%  scope %DHCPBEREICHSNAME% add reservedip %IP% %MAC% %HOSTNAME% "%KOMMENTAR%" "BOTH" >%ERRORLOG%
if not ERRORLEVEL 1 (
    echo -- Eintrag erfolgreich >>"%TEMPLOG%"
    set RES_STATUS="ERFOLG"
    goto Ausstieg
) ELSE (
    REM ---Fehlermeldung konservieren---
    type %ERRORLOG% > %ERRORLOG2%
)

call :ASKFIXMAC %IP% TESTMAC
:: gibt es zur IP eine MAC mit fixer Lease
set tempmak=0
call :MAKELOWMAC %MAC% TEMPMAK
set MAC=%TEMPMAK%
:: MAC in Kleinbuchstaben verwandeln
if "%TESTMAC%"=="%MAC%" (
     echo == RESERVIERUNG SCHON VORHANDEN: %IP% - %TESTMAC%     >>"%TEMPLOG%"
     set RES_STATUS="EXISTS"
) else (
     echo !! PROBLEM: Ein Fehler beim Eintragen ist aufgetreten >>"%TEMPLOG%"
     echo !! Eintragung nicht durchgefuehrt!                    >>"%TEMPLOG%"
     echo.                                                      >>"%TEMPLOG%"
     echo Fehlerhinweis von netsh:                              >>"%TEMPLOG%"
     type %ERRORLOG2%                                           >>"%TEMPLOG%"
     del %ERRORLOG2%
     set RES_STATUS="FEHLER"
)
::echo. >>"%TEMPLOG%"

:Ausstieg

::=====LOG==============================================================================
type "%TEMPLOG%" >> "%NEWLOG%"
type "%TEMPLOG%" >> "%LOGFILE%"
type "%TEMPLOG%" >> "%HOSTLOG%"
type "%TEMPLOG%"


if %RES_STATUS%=="FEHLER" (
  echo -- FEHLER: MAC-%MAC%  IP-%IP%  HOST-%HOSTNAME%    >>"%LOG_FAILED%"
)
if %RES_STATUS%=="ERFOLG" (
  echo -- ERFOLG: MAC-%MAC%  IP-%IP%  HOST-%HOSTNAME%    >>"%LOG_SUCCESS%"
)

if %RES_STATUS%=="EXISTS" (
  echo -- EXISTIERT : MAC-%MAC%  IP-%IP%  HOST-%HOSTNAME% >>"%LOG_EXIST%"
)
if %RES_STATUS%=="INACTIVE" (
  echo -- VORGEMERKT: MAC-%MAC%  IP-%IP%  HOST-%HOSTNAME% >>"%LOG_EXIST%"
)


:EXIT_ON_COMMENT

endlocal 
goto :EOF
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:ASKMAC HOSTNAME NEWMAC 
::
:: Func: Fragt den DHCP-Server nach der Mac-Addresse eines bestimmten PCs.
::       Der PC muss dazu schon eine Lease zugewiesen bekommen haben! 
::       Server 2000/2003.
:: 
:: Args: %1 Hostname
::       %2 NEWMAC-Adresse     
::   
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
::     Rechnername in IP-Adresse aufgelsen (NSLOOKUP), dann durch Abfrage
::
::     der Adressleases des DHCP-Servers die dazugehrige MAC-Adresse des PCs
::
::     ermittelt.
::
::     Voraussetzungen:
::
::     - Windows 2000/2003 Server als DHCP-Server
::
::     - Angemeldeter Benutzer muss Mitglied der Gruppe "DHCP-Benutzer" auf dem
::
::       DHCP-Server sein, um die DHCP-Daten auslesen zu knnen.
::
:: ************************************************************************************
setlocal
:: Variable loeschen
set IPAdresse=0.0.0.0
set MAC=00-00-00-00-00-00-00-00
set ZIELRECHNER=%1
if exist %ERRORLOG% del %ERRORLOG% 
nslookup %Zielrechner% > %TEMPDATEI1%  2>%ERRORLOG%

:: ---- Ausgabe z.B.:------------------------
::
::    Server:  server.schule.aps
::
::    Address:  192.168.100.200
::
::
::    Name:    PC01.schule.aps
::
::    Address:  192.168.100.1
:: ------------------------------------------

:: ---erste Zeile auslesen---
if exist %ERRORLOG% (
  set ERR_LINE=
  for /f "delims=" %%a in ('more^<%ERRORLOG%') DO (
     if not defined ERR_LINE set ERR_LINE=%%a
  )
)

:: ---Fehlermeldung ausgeben----
set HILF=%ERR_LINE:~0,3%
if "%HILF%x"=="***x" (
  echo -- %ERR_LINE:~4,80% >>"%TEMPLOG%"
  REM  Ergibt PC nicht gefunden
  echo !! PROBLEM: %HOSTNAME% beim DNS-Server unbekannt >>"%TEMPLOG%"
  goto EndeSub
)



type %TEMPDATEI1% | find /i /n "Address" | find /v "[2]Address" >%TEMPDATEI2%

::---- Ausgabe des 1. FIND-Befehls ----------
::
::        [2]Address:  192.168.100.200
::
::        [5]Address:  192.168.100.1
::
::---- Ausgabe des 2. FIND-Befehls ----------
::
::        [5]Address:  192.168.100.1
::
:: ------------------------------------------


for /f "tokens=2 delims=, " %%a in (%Tempdatei2%) DO set IPAdresse=%%a
echo -- IPAdresse '%IPAdresse%' fuer '%1' gefunden!  >>"%TEMPLOG%"


::--------- DNS auf doppelte Eintraege kontrollieren ----------------
::
:: 2 x nslookup fuer die gleiche IP-Adresse aufrufen
:: Bei DNS-Leichen werden dann 2 unterschiedliche Hostnamen angegeben
::

:: -- erster Aufruf -------------------------------------------------
nslookup %IPAdresse% > %TEMPDATEI1% 2>%ERRORLOG%

:: ---- Ausgabe z.B.:------------------------
::
::    Server:  server.schule.aps
::
::    Address:  192.168.100.200
::
::
::    Name:    vorarlbe-ac09ec
::
::    Address:  192.168.100.1
:: ------------------------------------------

type %TEMPDATEI1% | find /i  "Name" >%TEMPDATEI2%
for /f "tokens=2 delims= " %%a in (%Tempdatei2%) DO set PC_Name1=%%a
::echo PCNAME: %pc_Name1%
::

:: -- zweiter Aufruf -------------------------------------------------
nslookup %IPAdresse% > %TEMPDATEI1% 2>%ERRORLOG%

:: ---- Ausgabe z.B.:------------------------
::
::    Server:  server.schule.aps
::
::    Address:  192.168.100.200
::
::
::    Name:    PC01
::
::    Address:  192.168.100.1
:: ------------------------------------------

type %TEMPDATEI1% | find /i  "Name" >%TEMPDATEI2%
for /f "tokens=2 delims= " %%a in (%Tempdatei2%) DO set PC_Name2=%%a

if not "%PC_NAME1%"=="%PC_NAME2%" (
      echo !! PROBLEM: Der DNS-Server ist inkonsistent!                       >>"%TEMPLOG%"
      echo !!          Dies ergibt SCHWERE Probleme bei der Namensaufloesung! >>"%TEMPLOG%"
      echo !! =====^>   Bitte FORWARD- und REVERSE-LOOKUP-Zone ueberpruefen!  >>"%TEMPLOG%"
      echo !!          Gleiche IP-Adresse %IPAdresse% fuer min. 2 Hosts       >>"%TEMPLOG%"
      echo !!          - %PC_NAME1%                     >>"%TEMPLOG%"
      echo !!          - %PC_NAME2%                     >>"%TEMPLOG%" 
      exit /B 2         
      REM  beendet Batch und setzt ERRORLEVEL 2
) else (
      echo -- reverse Lookup fuer %IPAdresse% erfolgreich!                    >>"%TEMPLOG%"
)

:: --------------------------------------
:: -- Ermittle die MAC-Adresse ...
netsh dhcp server \\%DHCPServer% scope %DHCPBereichsname% show clients | find /i "%IPAdresse% " >%Tempdatei2%

:: ----------------------------------------------------------------------------------------------   
::  Gibt eine Liste aller Adressleases mit IP-Adr. + MAC-Adr. der PCs aus,
::
::  die mit Hilfe von FIND nach der betreffenden IP-Adresse durchsucht wird.
::
::  Ausgabe z.B.
::
::      192.168.100.12     - 255.255.255.0  - 00-30-84-6d-a4-12   -21.05.2007 07:23:28    -D
::      192.168.100.12     - 255.255.255.0  -00-30-84-6d-a4-12   -21.05.2007 07:23:28    -D
::  
:: INFO: aus ungeklaerter Ursache entspricht die Ausgabe manchmal der 2. Zeile !! (Update?)
::       Daher ist im folgenden eine weitere Ueberpruefung noetig!!
:: ----------------------------------------------------------------------------------------------

::----------------------------------------------FEHLER-----------
if errorlevel 1 exit /B 1
:: beendet Batch und setzt ERRORLEVEL 1

:: --- Mac-Adresse zusammenbauen ---
for /f "tokens=5 delims= " %%a in (%Tempdatei2%) DO set MAC=%%a
::for /f "tokens=5 delims= " %%a in ('more ^<%Tempdatei2%') DO set MAC=%%a
set MAC=%MAC:~0,2%%MAC:~3,2%%MAC:~6,2%%MAC:~9,2%%MAC:~12,2%%MAC:~15,2%


:: -----Problem ----
REM --wegen Problem mit fehlendem Leerzeichen in Ausgabe  ----
REM   sind folgende Zeilen noetig
REM zerlegt die Ausgabe anders
if NOT "%MAC%"=="-" goto MAC_WEITER
  REM  for /f "tokens=4 delims= " %%b in ('more ^<%Tempdatei2%') DO set MAC=%%b
  for /f "tokens=4 delims= " %%a in (%Tempdatei2%) DO set MAC=%%a
  set MAC=%MAC:~1,2%%MAC:~4,2%%MAC:~7,2%%MAC:~10,2%%MAC:~13,2%%MAC:~16,2%
:mac_weiter
::--Problem - Ende ----


echo -- MACAdresse '%MAC%' gefunden!   >>"%TEMPLOG%"

:EndeSub
endlocal & set "%2=%MAC%"
:: (&) expandiert zuerst Zeile und verarbeitet dann alles
goto :eof





:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:ASKFIXMAC IP NEWMAC 
::
::
:: Func: Fragt den DHCP-Server nach der Mac-Addresse eines bestimmten PCs.
::       Der PC muss dazu schon eine FIXE Lease zugewiesen bekommen haben! 
::       Server 2000/2003.
:: 
:: Args: %1 IP
::       %2 NEWMAC-Adresse (Rueckgabe by Ref)    
::   
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
setlocal 
:: ---- Variable loeschen ---
set IPAdresse=%1
set MAC=00-00-00-00-00-00-00-00
:: ---- Ermittle die MAC-Adresse ...
::netsh dhcp server \\%DHCPServer% scope %DHCPBereichsname% show clients 1
::pause
netsh dhcp server \\%DHCPServer% scope %DHCPBereichsname% show reservedip | find /i "%IPAdresse% " >%Tempdatei2% 2>%ERRORLOG%

:: ----------------------------------------------------------------------------------------------   
::  Gibt eine Liste aller Reservierungsadressen mit IP-Adr. + MAC-Adr. der PCs aus,
::
::  die mit Hilfe von FIND nach der betreffenden IP-Adresse durchsucht wird.
::
::  Ausgabe z.B.
::
::      192.168.100.240   -    05-0e-0c-9c-12-9d
::
:: Anzahl der Reservierten IP-Bereiche im Bereich 192.168.100.0: 1.
:: ----------------------------------------------------------------------------------------------

::----------------------------------------------FEHLER-----------
if errorlevel 1 goto raus

:: --- Mac-Adresse zusammenbauen ---
for /f "tokens=3 delims= " %%a in (%Tempdatei2%) DO set MAC=%%a
set MAC=%MAC:~0,2%%MAC:~3,2%%MAC:~6,2%%MAC:~9,2%%MAC:~12,2%%MAC:~15,2%

:raus
endlocal & set "%2=%MAC%"
:: (&) expandiert zuerst Zeile und verarbeitet dann alles
goto :eof
::----- END-ASKFIXMAK-------------------------------------------------------------------------------
:: *************************************************************************************************







:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:MakeLowMAC MAC LOWMAC 
::
:: Func: Konvertiert Mac-Addresse in Kleinbuchstaben.
:: 
:: Args: %1 Original-MAC
::       %2 LOWMAC-Adresse     
::   
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
setlocal 
set MAC=%1
set MAC=%MAC:A=a%
set MAC=%MAC:B=b%
set MAC=%MAC:C=c%
set MAC=%MAC:D=d%
set MAC=%MAC:E=e%
set MAC=%MAC:F=f%
endlocal & set "%2=%MAC%"
rem (&) expandiert zuerst Zeile und verarbeitet dann alles
goto :eof
::--------------------------------------------------------






::-----------------------------------------------------------------------------------------
:ErrorNoServer
color CF
:: ====LOG=============================================================================
echo.                                                                        > "%TEMPLOG%"
echo.                                                                       >> "%TEMPLOG%"
echo ====================================================================== >> "%TEMPLOG%"
echo -- PROBLEM --    Das Programm muss auf dem Server ausgefuehrt werden!  >> "%TEMPLOG%"
echo ====================================================================== >> "%TEMPLOG%"
echo.                                                                       >> "%TEMPLOG%"
echo Das Programm muss wegen "netsh" auf dem Server ausgefuehrt werden!      >> "%TEMPLOG%"
echo.                                                                       >> "%TEMPLOG%"
echo.------------------------------------------------------------------     >> "%TEMPLOG%"
echo.                                                                       >> "%TEMPLOG%"
type "%TEMPLOG%" >>"%NEWLOG%"
type "%TEMPLOG%" >>"%LOGFILE%"
type "%TEMPLOG%"
:: ====LOG=============================================================================
goto ende
::-----------------------------------------------------------------------------------------



::-----------------------------------------------------------------------------------------
:ErrorNoFile
color CF
:: ====LOG=============================================================================
echo.                                                                        > "%TEMPLOG%"
echo.                                                                       >> "%TEMPLOG%"
echo ====================================================================== >> "%TEMPLOG%"
echo -- PROBLEM --       Datei nicht gefunden!                              >> "%TEMPLOG%"
echo ====================================================================== >> "%TEMPLOG%"
echo.                                                                       >> "%TEMPLOG%"
echo - Datei %myfile% nicht gefunden !!!                                    >> "%TEMPLOG%"
echo.                                                                       >> "%TEMPLOG%"
echo   In dieser Datei sind die gewuenschten Reservierungen eingetragen!    >> "%TEMPLOG%"
echo   Ein Muster als Kopiervorlage ist im Quelltext dieser Batch!          >> "%TEMPLOG%"
echo.                                                                       >> "%TEMPLOG%"
type "%TEMPLOG%" >>"%NEWLOG%"
type "%TEMPLOG%" >>"%LOGFILE%"
type "%TEMPLOG%"
:: ====LOG=============================================================================
goto ende
::-----------------------------------------------------------------------------------------



:ende
pause


goto eof

exit
:: nur zur Sicherheit





=======================================================================
-----------------------------------------------------------------------
Muster fuer die Datei "fixlease_list.txt"
-----------------------------------------------------------------------
=======================================================================

Dieses Muster kann kopiert, angepasst und dann
unter dem Namen "fixlease_list.txt" gespeichert werden.
Pfad kann innerhalb der Batch bei 
"Server-Variable setzen" geaendert werden!

---SCHNIPP-------BEGINN-MUSTER-----------------------------------------

*****************************************************
*                                                   *
*    Config File fuer DHCP-Reservierungen           *
*                                                   *
*    ------------------------                       *
*    MAC;IP;HOST;BESCHREIBUNG                       *
*    ------------------------                       *
*    Struktur der Tabelle bitte nicht ndern!       *
*                                                   *
*    - Trennung durch genau 3 Strichpunkte          *
*    - kein Strichpunkt nach BESCHREIBUNG           *
*    - keine Leerzeichen                            *
*    - BESCHREIBUNG darf fehlen:                    *
*      dann wird HOST als BESCHREIBUNG verwendet    *
*    - nach "HOST"-Eintrag: mit Semikolon           *
*                                                   *
*    - neue Eintraege: Liste ergaenzen oder         *
*                      Liste neu erstellen          *
*                                                   *
*                                                   *
* --HINWEIS-----------------------------------------*
*                                                   *
*   Die fix vergebenen Adressen drfen NICHT im     *
*   Pool der dynamischen vom DHCP-Server            *
*   vergebenen Bereich liegen!                      *
*                                                   *
*   Daher starten die untenstehenden Eintraege      *
*   erst mit der IP 192.168.101.101.                *
*   Der dynamische Bereich liegt beim Musterserver  *
*   der Pflichtschulen zwischen                     *
*   192.168.101.1  und  192.168.101.254             *
*                                                   *
* --Beispiele---------------------------------------*
*                                                   *
* 030E0C9A11B5;192.168.101.3;PC03;EDV1              * 
*      MAC         IP          HOST BESCHREIBUNG    *
* Reservierung mit dieser MAC und IP                *
* Hostname ist PC03 und Beschreibung ist EDV1       *
*                                                   *
*---------------------------------------------------*
* 030E0C9A11A5;192.168.101.1;PC01;                  * 
*                                 ohne BESCHREIBUNG *
* Hostname ist PC01 und Beschreibung ist PC01       *
*                                                   *
*---------------------------------------------------*
* 000000000000;192.168.101.1;PC0815;EDV2            *
*   NULL-MAC                                        *
* richtige MAC von PC0815 wird beim DHCP-Server     *
* automatisch ermittelt (netsh)                     *
* Reservierung mit gefundener MAC + vorgegebener IP *
*                                                   *
*---------------------------------------------------*
* 050E0C9C129D;192.168.101.4;PC05;LZ                *
* 000000000000;192.168.101.5;PC88;                  * 
* 060E0C9C11B3;192.168.101.6;PC06;EDV2              *
* 060E0C9C15B3;192.168.101.7;PC07;                  *
*                                                   *
*****************************************************
*                                                   *
*  Stern (*) in ERSTER Spalte ergibt Kommentarzeile *
*                                                   *
*****************************************************
*
*-----------
* EDV-Saal-1
*-----------
000000000000;192.168.101.1;PC01;EDV1
000000000000;192.168.101.2;PC02;EDV1
000000000000;192.168.101.3;PC03;EDV1
000000000000;192.168.101.4;PC04;EDV1
000000000000;192.168.101.5;PC05;EDV1
000000000000;192.168.101.6;PC06;EDV1
000000000000;192.168.101.7;PC07;EDV1
000000000000;192.168.101.8;PC08;EDV1
000000000000;192.168.101.9;PC09;EDV1
000000000000;192.168.101.10;PC10;EDV1
000000000000;192.168.101.11;PC11;EDV1
000000000000;192.168.101.12;PC12;EDV1
000000000000;192.168.101.13;PC13;EDV1
000000000000;192.168.101.14;PC14;EDV1
000000000000;192.168.101.15;PC15;EDV1
*-----------
* Lehrer-PCs
*-----------
*C00000000002;192.168.101.100;PCL01;Lehrer01
*-----------
* Drucker
*-----------
*050E0C9C129D;192.168.100.240;LaserJ;PRN-EDV1


---SCHNAPP-------ENDE-MUSTER-----------------------------------------

:eof

